package com.boman.auth.controller;

import javax.servlet.http.HttpServletRequest;

import com.alibaba.fastjson.JSONObject;
import com.boman.common.core.exception.BaseException;
import com.boman.domain.SysUser;
import com.boman.domain.WechatEnum;
import com.boman.domain.dto.AjaxResult;
import com.boman.domain.form.LoginBody;
import com.boman.wechat.api.RemoteWechatService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.boman.auth.service.SysLoginService;
import com.boman.domain.dto.R;
import com.boman.common.core.utils.StringUtils;
import com.boman.common.security.service.TokenService;
import com.boman.system.api.model.LoginUser;

import java.util.Map;

import static com.boman.common.core.utils.SecurityUtils.checkStrongPwd;

/**
 * token 控制
 *
 * @author ruoyi
 */
@RestController
//@CrossOrigin
public class TokenController {

    @Autowired
    private TokenService tokenService;

    @Autowired
    private SysLoginService sysLoginService;

    @Autowired
    private RemoteWechatService remoteWechatService;

    @PostMapping("login")
    public R<?> login(@RequestBody LoginBody form) {
        String openId = "";
        // 验证是pc还是小程序
        if (form.getLogintype() != null && form.getLogintype().equals(WechatEnum.W.getValue())) {
            AjaxResult result = remoteWechatService.getWechatInfo(form);
            if ((Integer) result.get(AjaxResult.CODE_TAG) != 0) {
                throw new BaseException((String) result.get(AjaxResult.MSG_TAG));
            }
            Map<String, Object> jsonObject = (Map<String, Object>) result.get(AjaxResult.DATA_TAG);
            if (jsonObject == null) {
                throw new BaseException("未获取到微信鉴权相关信息！");
            }
            openId = (String) jsonObject.get("openId");
        }
         if ("1".equals(checkStrongPwd(form.getPassword()))) {
            return R.fail("密码必须包含数字、大小写字母、特殊符号且大于8位");
        }
        // 用户登录
        LoginUser userInfo = sysLoginService.login(form.getUsername(), form.getPassword());
        // 如果是微信登录并且获取到的用户的unionId不为空
        if (form.getLogintype() != null
                && form.getLogintype().equals(WechatEnum.W.getValue())
                && StringUtils.isNotEmpty(openId)) {
            SysUser sysUser = new SysUser();
            sysUser.setId(userInfo.getSysUser().getId());
            sysUser.setOpenId(openId);
            AjaxResult updateResult = sysLoginService.updateUnionId(sysUser);
            if((Integer) updateResult.get(AjaxResult.CODE_TAG) != 0) {
                throw new BaseException("保存小程序唯一标识出错！");
            }
        }
        // 获取登录token
        return R.ok(tokenService.createToken(userInfo));
    }

    @DeleteMapping("logout")
    public R<?> logout(HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (StringUtils.isNotNull(loginUser)) {
            String username = loginUser.getUsername();
            // 删除用户缓存记录
            tokenService.delLoginUser(loginUser.getToken());
            // 记录用户退出日志
            sysLoginService.logout(username);
        }
        return R.ok();
    }

    @PostMapping("refresh")
    public R<?> refresh(HttpServletRequest request) {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (StringUtils.isNotNull(loginUser)) {
            // 刷新令牌有效期
            tokenService.refreshToken(loginUser);
            return R.ok();
        }
        return R.ok();
    }
}
